editor = {
    active = false,
    toolbar = {
        x = 10,
        y = 60,
        cols = 1,
        buttons = {},
        scale = 2,
        grabbed = false,
        grabX, grabY,
        titleh = 8,
        title = "Tile"
    },
    curtile = 1,
    loaded = false,
    focus = "",
    mapexists = false,
    unsaved = false
}

function editor_load()
    editor.toolbar.w = editor.toolbar.cols * screen.tilesize + 4
    editor.toolbar.h = editor.toolbar.titleh + math.ceil(#game.tilequad / editor.toolbar.cols) * screen.tilesize + 3
    editor.toolbar.canvas = love.graphics.newCanvas(editor.toolbar.w, editor.toolbar.h)
    editor.toolbar.canvas:clear(0,0,0,150)
    editor.toolbar.canvas:setFilter("nearest", "nearest")

    love.graphics.push()
    love.graphics.translate(2,1)
    love.graphics.setCanvas(editor.toolbar.canvas)
    drawShadowText(editor.toolbar.title, 0, 1, editor.toolbar.w-2, "center", font.tiny)
    local x, y = 0, 0
    editor.toolbar.buttons = {}
    for i = 1, #game.tilequad do
        local xx, yy = x * screen.tilesize, y * screen.tilesize + editor.toolbar.titleh
        love.graphics.setColor(255,255,255)
        love.graphics.draw(game.tiles, game.tilequad[i], xx, yy)
        if game.thit[i] == 1 then
            love.graphics.setColor(255,0,0)
            love.graphics.circle("fill", xx + 3, yy + 3, 2)
        end
        editor.toolbar.buttons[#editor.toolbar.buttons+1] = { x = xx, y = yy, w = screen.tilesize, h = screen.tileheight, tile = i }
        x = x + 1
        if x >= (editor.toolbar.cols) then
            y = y + 1
            x = 0
        end
    end
    love.graphics.setColor(255,255,255)
    love.graphics.rectangle("line", editor.toolbar.buttons[editor.curtile].x+.5, editor.toolbar.buttons[editor.curtile].y+.5, screen.tilesize-1, screen.tilesize-1)
    love.graphics.pop()
    love.graphics.setColor(255,255,255)
    love.graphics.rectangle("line", .5, .5, editor.toolbar.w-1, editor.toolbar.h-1)
    love.graphics.setCanvas()

    editor.loaded = true
end

function editor_update(dt)
    local msx, msy = love.mouse.getX(), love.mouse.getY()
    if editor.loaded == false then
        editor_load()
    end

    if editor.active then
        if love.mouse.isDown("l") == false then
            if editor.toolbar.grabbed == true then editor.toolbar.grabbed = false end
            editor.focus = ""
        end
        if love.mouse.isDown("l") then
            if editor.toolbar.grabbed == true then
                editor.toolbar.x = msx - editor.toolbar.grabX
                editor.toolbar.y = msy - editor.toolbar.grabY
            elseif editor.toolbar.grabbed == false then
                if mouseInToolbar(msx, msy, editor.toolbar) and editor.focus ~= "map" then
                    editor.focus = "tools"

                    if msy <= editor.toolbar.y + (editor.toolbar.titleh * editor.toolbar.scale) then
                        if editor.toolbar.grabbed == false then
                            editor.toolbar.grabX = msx - editor.toolbar.x
                            editor.toolbar.grabY = msy - editor.toolbar.y
                            editor.toolbar.grabbed = true
                        end
                    end
                elseif editor.focus ~= "tools" then
                    editor.focus = "map"
                    local tx, ty = math.floor(mx / screen.tilesize), math.floor(my / screen.tilesize)
                    local t = editor.curtile
                    game.map[tx][ty][2] = t
                    game.map[tx][ty][1] = game.thit[t]
                    editor.unsaved = true
                end
            end
        end
    end
end

function editor_draw()
    local tx, ty = math.floor(mx / screen.tilesize), math.floor(my / screen.tilesize)
    if editor.active and screen.panning == "" then
        for x = 0, 21 do
            for y = 0, 13 do
                if game.map[x][y][1] == 1 then
                    love.graphics.setColor(255,0,0)
                    love.graphics.rectangle("line",
                        x * (screen.tilesize * screen.scale) - (screen.tilesize/2*screen.scale) +.5,
                        y * (screen.tilesize * screen.scale) + .5,
                        (screen.tilesize * screen.scale) - 1,
                        (screen.tilesize * screen.scale) - 1
                    )
                end
            end
        end

        love.graphics.setColor(255,255,255,200)
        love.graphics.draw(game.tiles, game.tilequad[editor.curtile], tx * (screen.tilesize * screen.scale) - (screen.tilesize/2*screen.scale), ty * (screen.tilesize * screen.scale), 0, screen.scale, screen.scale)

        love.graphics.setColor(255,255,255)
        love.graphics.rectangle("line",
            tx * (screen.tilesize * screen.scale) - (screen.tilesize/2*screen.scale) + .5,
            ty * (screen.tilesize * screen.scale) + .5,
            (screen.tilesize * screen.scale) - 1,
            (screen.tilesize * screen.scale) - 1
        )

        if editor.focus ~= "map" then
            love.graphics.setColor(255,255,255)
            love.graphics.draw(editor.toolbar.canvas, editor.toolbar.x, editor.toolbar.y, 0, editor.toolbar.scale, editor.toolbar.scale)
        end
        local t1, t2 = "", ""
        if editor.mapexists == false then
            t1 = "This map hasn't been saved yet.  "
        end
        if editor.unsaved == true then
            t2 = "There are UNSAVED changes. Press S to save this map.  "
        end
        drawShadowText(t1 .. t2, 5, love.graphics.getHeight() - 14, 500, "left", font.dialog)
    end
end

function editor_save()
    editor.unsaved = false
    editor.mapexists = true
end

function editor_keypressed(k)
    if editor.active then
        if shiftHeld() then
            if k == "left" then
                screen:pan(k, false)
            elseif k == "right" then
                screen:pan(k, false)
            elseif k == "up" then
                screen:pan(k, false)
            elseif k == "down" then
                screen:pan(k, false)
            end
        end
        if k == "s" then
            editor_save()
        end
        if k == "\\" then
            local a = 1
            if shiftHeld() then a = -1 end
            editor.toolbar.cols = editor.toolbar.cols + a
            if editor.toolbar.cols > math.min(#game.tilequad, 10) then editor.toolbar.cols = 1 end
            if editor.toolbar.cols < 1 then editor.toolbar.cols = math.min(#game.tilequad, 10) end
            editor.loaded = false
        end
    end
end

function editor_mousepressed(x, y, b)
    if editor.active then
        local tx, ty = editor.toolbar.x, editor.toolbar.y
        local ts = editor.toolbar.scale
        if b == "l" then
            if shiftHeld() then
                local tx, ty = math.floor(mx / screen.tilesize), math.floor(my / screen.tilesize)
                player[1].x = tx
                player[1].y = ty
            else
                for i, btn in pairs(editor.toolbar.buttons) do
                    if x >= tx + btn.x * ts and x < tx + (btn.x + btn.w) * ts and y >= ty + btn.y * ts and y < ty + (btn.y + btn.w) * ts then
                        editor.curtile = btn.tile
                        editor.loaded = false
                    end
                end
            end
        end

        if b == "wu" then
            editor.curtile = editor.curtile + 1
            if editor.curtile > #game.tilequad then editor.curtile = #game.tilequad end
            editor.loaded = false
        elseif b == "wd" then
            editor.curtile = editor.curtile - 1
            if editor.curtile < 1 then editor.curtile = 1 end
            editor.loaded = false
        end
    end
end

function shiftHeld()
    return love.keyboard.isDown("lshift") or love.keyboard.isDown("rshift")
end

function drawShadowText(t, x, y, w, a, f)
    love.graphics.setFont(f)
    love.graphics.setColor(0,0,0)
    love.graphics.printf(t, x+1, y+1, w, a)
    love.graphics.setColor(255,255,255)
    love.graphics.printf(t, x, y, w, a)
end

function mouseInToolbar(x, y, tb)
    if x >= tb.x and x < tb.x + (tb.w * tb.scale) and y >= tb.y and y < tb.y + (tb.h * tb.scale) then
        return true
    end
    return false
end

