local fLeft, fUp, fRight, fDown = 1, 2, 3, 4
local dStr = {"Left", "Up", "Right", "Down"}

enemyType = {}

function enemyType:create(...)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:init(...)
    return instance
end
function enemyType:init(x, y, id, v)
    v = v or {}
    self.x = x
    self.y = y
    self.px = x
    self.py = y
    self.type = math.random(1,4)
    self.speed = math.random(1,4) + math.random(1,10) / 10
    self.id = id
    self.uuid = createUUID()
    self.dropcoin = v.dropcoin or false
    self.moving = ""
    self.facing = "down"
    self.frame = 1
    self.ox = 0
    self.oy = 0
    self.tx = 0
    self.ty = 0
    self.haspath = false
    self.path = nil
    self.pathstr = ""
    self.pathcur = 0
    self.pathnum = 0
    self.canbepushed = false
    self:wait(.25)
    self.co = nil
    self.costat = nil
    self:activate()
end

function enemyType:update(dt)
    if self.moving ~= "" then self.frame = (self.frame + dt * self.speed * 2) % 2 end

    behavior.pathfind(self, dt)
end

function enemyType:draw()
    local x = math.floor(self.x * screen.tilesize - (self.ox * screen.tilesize) + (screen.tilesize / 2))
    local y = math.floor(self.y * screen.tilesize - (self.oy * screen.tilesize) + (screen.tilesize / 2))

--    love.graphics.setColor(200,180,0)
--    love.graphics.line((self.tx + .5) * screen.tilesize, (self.ty + .5) * screen.tilesize, x, y)
--    love.graphics.circle("fill", (self.tx + .5) * screen.tilesize, (self.ty + .5) * screen.tilesize, 4)

    drawpool[#drawpool+1] = {
        type = "image", x = x, y = y + 4,
        i = game.enemyimg, q = game.enemyquad[self.type][math.floor(self.frame)][self.facing],
        ox = 8, oy = 16 + game.map[self.x][self.y][3], who = "enemy"
    }

    if math.floor(mx / screen.tilesize) == self.x and math.floor(my / screen.tilesize) == self.y then
        love.graphics.setFont(font["dialog"])
        local t = ""
        --[[t = t .. self.doing .. " (" .. self.x .. "," .. self.y .. " - " .. self.ox .. "," .. self.oy .. ")\n"
        t = t .. "Has path: " .. tostring(self.haspath) .. "\n"
        if self.doing == "pursue" and self.path then
            t = t .. "Status: " .. coroutine.status(self.co) .. "\n"
            t = t .. tostring(self.pathcur) .. " of " .. tostring(self.pathnum) .. "\n"
            t = t .. tostring(self.path[self.pathcur]) .. "\n"
            t = t .. self.pathstr .. "\n"
        elseif self.doing == "wait" then
            t = t .. self.waittime .. "\n"
        end]]
        for i, p in pairs(self) do
            if type(p) ~= "table" then
                t = t .. i .. " = " .. tostring(p) .. "\n"
            else
                t = t .. i .. ":\n"
                t = t .. "  "
                for j, q in pairs(p) do
                    t = t .. tostring(q) .. ","
                end
                t = t .. "\n"
            end
        end
        debugger.enemytext = t
    end
end


function enemyType:left()
    if self:canMove(-1,0) then
        self.ox = -1
        self.moving = "left"
        self.facing = "left"
        self.px = self.x
        self.x = self.x - 1
    else
        self:stop()
    end
end
function enemyType:right()
    if self:canMove(1,0) then
        self.ox = 1
        self.moving = "right"
        self.facing = "right"
        self.px = self.x
        self.x = self.x + 1
    else
        self:stop()
    end
end
function enemyType:up()
    if self:canMove(0,-1) then
        self.oy = -1
        self.moving = "up"
        self.facing = "up"
        self.py = self.y
        self.y = self.y - 1
    else
        self:stop()
    end
end
function enemyType:down()
    if self:canMove(0,1) then
        self.oy = 1
        self.moving = "down"
        self.facing = "down"
        self.py = self.y
        self.y = self.y + 1
    else
        self:stop()
    end
end
function enemyType:stop()
    --print(self.id, self.pathcur, self.pathnum)
    self.moving = ""
    self.ox = 0
    self.oy = 0
    self.px = self.x
    self.py = self.y
    self.pathcur = self.pathcur + 1
    if self.pathcur > self.pathnum then
        --print(self.id, "Finished path")
        self.path = nil
        self.pathstr = ""
        self.pathcur = 0
        self.pathnum = 0
        self.haspath = false
        self:wait(math.random(1,3))
    end
    if self.dropcoin then
        local i = #item+1
        item[i] = itemType:create(self.x, self.y, i, { anim = 0 } )
    end
end

function enemyType:wait(t)
    self.waittime = t
    self.doing = "wait"
end

function enemyType:canMove(ox, oy)
    local x, y = self.x+ox, self.y+oy
    return not blocked(x, y, self.uuid)
end

function enemyType:activate()
    self.active = true
    if self.solidwhenactive then
        self.solid = true
    end
    print("Activating ENEMY " .. self.id)
end

function enemyType:deactivate()
    self.active = false
    if self.solidwhenactive then
        self.solid = false
    end

    print("Deactivating ENEMY " .. self.id)
end

