function drawComposite()
    drawpool = {}

    for x = 0, 20 do
        for y = 0, 15 do
            local t = game.map[x][y][2]
            local h = game.map[x][y][3]
            if t == 0 then t = 1 end
            if t == 4 then
                h = -3 + (math.cos(love.timer.getTime() * 5 + (x + y)) * 1)
            end
            drawpool[#drawpool+1] = {
                type = "image", x = x * screen.tilesize, y = y * screen.tilesize,
                i = game.tiles, q = game.tilequad[t],
                ox = 0, oy = math.floor(h), who = "tile_" .. t
            }
        end
    end

    for _, i in pairs(item) do
        i:draw()
    end

    for _, e in pairs(enemy) do
        e:draw()
    end

    for _, n in pairs(npc) do
        n:draw()
    end

    for _, p in pairs(player) do
        p:draw()
    end

    sort(drawpool)

    love.graphics.setCanvas(screen.canvas["main"])
    screen.canvas["main"]:clear(0,0,0,0)
    love.graphics.setBackgroundColor(0,0,0)
    love.graphics.setColor(255,255,255)

    love.graphics.push()
    love.graphics.translate(0, 8)

    for i, d in pairs(drawpool) do
        if d.type == "image" then
            love.graphics.setColor(255,255,255)
            love.graphics.draw(d.i, d.q, d.x, d.y, 0, 1, 1, d.ox, d.oy)
        elseif d.type == "line" then
            love.graphics.setColor(d.c)
            love.graphics.line(d.x, d.y, d.x2, d.y2)
        else
            love.graphics.setColor(d.c)
            love.graphics.circle("fill", d.x, d.y, d.r, 20)
        end
        --print(d.x/16, d.y/16, d.i, d.q, d.x, d.y, d.ox, d.o, d.who)
        --love.graphics.setColor(255,0,0)
        --love.graphics.circle("fill", d.x, d.y + 8, 1)
    end

    love.graphics.pop()

    scriptDraw()

    love.graphics.setCanvas()
end

function createGrid(sx, sy, w, h, v)
    v = v or 0
    local g = {}
    for x = sx, w-1 do
        g[x] = {}
        for y = sy, h-1 do
            g[x][y] = v
        end
    end
    return g
end

function newGame()
    print("Starting new game")
    game.state = "game"
    game.paused = false
    player = {
        [1] = playerType:create(10, 10, 1),
        --[2] = playerType:create(9, 1, 2)
    }

    npc = {
        [1] = npcType:create(6, 10, 1, { name = "Elizabeth", mapx = 1, mapy = 1, type = 1 }),
        [2] = npcType:create(4, 7, 1, { name = "Hayley", mapx = 1, mapy = 1, type = 2 }),
        [3] = npcType:create(7, 8, 1, { name = "Josie", mapx = 1, mapy = 2, type = 2 }),
        [4] = npcType:create(15, 9, 1, { name = "Katniss", mapx = 2, mapy = 1, type = 2 }),
        [5] = npcType:create(6, 10, 1, { name = "Olivia", mapx = 2, mapy = 2, type = 1 }),
    }

    loadScreen(player[1].mapx, player[1].mapy)
    player[1]:stop()
end

function loadGame()
    newGame()
end

function loadMap(x, y)
    local m
    if x == -1 and y == nil then
        m = love.filesystem.load("maps/map_template.lua")()
        editor.mapexists = false
        editor.unsaved = false
    else
        m = love.filesystem.load("maps/map_"..x.."_"..y..".lua")()
        editor.mapexists = true
        editor.unsaved = false
    end
    local m2 = createGrid(-2,-2,23,18, {0,0,0})
    for x = 1, #m do
        for y = 1, #m[x] do
            m2[x-1][y-1] = m[x][y]
            if m[x][y][3]== nil then
                local t = m2[x-1][y-1][2]
                if t == 0 then
                    print("NO!")
                    t = 1
                end
                local h = 0
                if t == 3 then h = 8 end
                if t == 2 then h = 16 end
                m2[x-1][y-1][3] = h--math.random(0,4)
            end
        end
    end

    print("\nLoad Map")

    return m2
end

function loadScreen(x, y)
    game.map = loadMap(x, y)

    enemy = {}
    for i = 1, 10 do
        enemy[i] = enemyType:create(math.random(1, 19), math.random(2,12), i)
    end

    item = {}
    local i = 0
    for x = 1, screen.w / screen.tilesize-1 do
        for y = 1, screen.h / screen.tilesize-1 do
            if blocked(x,y) == false and math.random(0,4) == 0 then
                i = i + 1
                item[i] = itemType:create(x, y, i, { anim = x + y / 2 } )
            end
        end
    end
end

function blocked(x, y, id)
    id = id or 0
    local cm = false

    if x < -2 then x = -2 end
    if y < -2 then y = -2 end
    if x > 22 then x = 22 end
    if y > 14 then y = 14 end

    if x == -2 or y == -2 or x == 22 or y == 14 then
        cm = true
        who = "bounds"
    end

    if game.map[x][y][1] == 1 then
        cm = true
        who = "wall"
    end

    for i, e in pairs(enemy) do
        if cm == false and e.uuid ~= id then
            if (x == e.x and y == e.y) or (x == e.px and y == e.py and math.abs(e.oy) <= .75 and math.abs(e.ox) <= .75) then
                cm = true
                who = "enemy"
            end
        end
    end

    for i, n in pairs(npc) do
        if cm == false and n.uuid ~= id and player[1].mapx == n.mapx and player[1].mapy == n.mapy then
            if (x == n.x and y == n.y) or (x == n.px and y == n.py and math.abs(n.oy) <= .75 and math.abs(n.ox) <= .75) then
                cm = true
                who = "npc"
            end
        end
    end

    for i, p in pairs(player) do
        if x == p.x and y == p.y and p.uuid ~= id then
            cm = true
            who = "player"
        end
    end

    --print(id, cm, who)
    return cm, who
end

function createTiles()
    local id = love.image.newImageData("images/tiles.png")
    local n = 1
    for y = 0, (game.tiles:getHeight()/(screen.tilesize*2))-1 do
        for x = 0, (game.tiles:getWidth()/screen.tilesize)-1 do
            game.tilequad[n] = love.graphics.newQuad(x * screen.tilesize, y * (screen.tilesize * 2), screen.tilesize, screen.tilesize * 2, game.tiles:getWidth(), game.tiles:getHeight())
            if game.thit[n] == nil then game.thit[n] = 0 end
            n = n + 1
        end
    end
end

function createItems()
    for i = 0, 1 do
        game.itemquad[i] = {}
        for q = 0, 7 do

        game.itemquad[i][q] = love.graphics.newQuad(q * screen.tilesize, i * screen.tilesize, screen.tilesize, screen.tilesize, game.itemimg:getWidth(), game.itemimg:getHeight())

        end
    end
end

function createActors()
    local dirs = { "left", "up", "right", "down" }
    for f = 0, 1 do
        game.playerquad[f] = {}
        for d = 1, 4 do
            game.playerquad[f][dirs[d]] = love.graphics.newQuad((d-1) * screen.tilesize, f * screen.tilesize, screen.tilesize, screen.tilesize, game.playerimg:getWidth(), game.playerimg:getHeight())
        end
    end

    for e = 1, 2 do
        game.npcquad[e] = {}
        for f = 0, 1 do
            game.npcquad[e][f] = {}
            for d = 1, 4 do
                game.npcquad[e][f][dirs[d]] = love.graphics.newQuad(
                    (d-1) * screen.tilesize, f * screen.tilesize + ((e-1) * screen.tilesize * 2),
                    screen.tilesize, screen.tilesize,
                    game.npcimg:getWidth(), game.npcimg:getHeight())
            end
        end
    end

    for e = 1, 4 do
        game.enemyquad[e] = {}
        for f = 0, 1 do
            game.enemyquad[e][f] = {}
            for d = 1, 4 do
                game.enemyquad[e][f][dirs[d]] = love.graphics.newQuad(
                    (d-1) * screen.tilesize, f * screen.tilesize + ((e-1) * screen.tilesize * 2),
                    screen.tilesize, screen.tilesize,
                    game.enemyimg:getWidth(), game.enemyimg:getHeight())
            end
        end
    end
end

function createUUID()
    local uuid = ""
    local chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    for i = 1, 10 do
        local l = math.random(1, #chars)
        uuid = uuid .. string.sub(chars, l, l)
    end
    --print("Generated UUID:", uuid)
    return uuid
end

function HSL(h, s, l, a)
    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

function sort(T) table.sort(T, function(a, b) return a.y < b.y end ) end

function roundrect(mode, x, y, w, h, rad)
    x = math.floor(x)
    y = math.floor(y)
    if w < rad*2 then rad = w / 2 end
    if h < rad*2 then rad = h / 2 end
    love.graphics.rectangle("fill", x+rad, y, w-rad*2, h)
    love.graphics.rectangle("fill", x, y+rad, rad, h-rad*2)
    love.graphics.rectangle("fill", x+w-rad, y+rad, rad, h-rad*2)
    love.graphics.arc("fill", x+rad, y+rad, rad, math.rad(179), math.rad(270))    --Top Left
    love.graphics.arc("fill", x+w-rad, y+rad, rad, math.rad(270), math.rad(360))    --Top Right
    love.graphics.arc("fill", x+w-rad, y+h-rad, rad, math.rad(0), math.rad(90))       --Bottom Right
    love.graphics.arc("fill", x+rad, y+h-rad, rad, math.rad(90), math.rad(181))     --Bottom Left
end
