npcType = {}

function npcType:create(...)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:init(...)
    return instance
end
function npcType:init(x, y, id, v)
    v = v or {}
    self.mapx = v.mapx or 1
    self.mapy = v.mapy or 1
    self.x = x
    self.y = y
    self.type = v.type or math.random(1,2)
    self.speed = 2
    self.id = id
    self.uuid = createUUID()
    self.ox = 0
    self.oy = 0
    self.moving = ""
    self.facing = "down"
    self.frame = 0
    self.haspath = false
    self.path = nil
    self.pathstr = ""
    self.pathcur = 0
    self.pathnum = 0
    self.canbepushed = false
    self:wait(.25)
    self.co = nil
    self.costat = nil
--    self:stop()
end

function npcType:update(dt)
    if player[1].mapx == self.mapx and player[1].mapy == self.mapy then
        if self.moving ~= "" then self.frame = (self.frame + dt * self.speed) % 2 end

        behavior.pathfind(self, dt)

        if self.moving ~= "" then self.facing = self.moving end
    end
end

function npcType:left()
    if self:canMove(-1,0) then
        self.ox = -1
        self.moving = "left"
        self.x = self.x - 1
    else
        self:stop()
    end
end
function npcType:right()
    if self:canMove(1,0) then
        self.ox = 1
        self.moving = "right"
        self.x = self.x + 1
    else
        self:stop()
    end
end
function npcType:up()
    if self:canMove(0,-1) then
        self.oy = -1
        self.moving = "up"
        self.y = self.y - 1
    else
        self:stop()
    end
end
function npcType:down()
    if self:canMove(0,1) then
        self.oy = 1
        self.moving = "down"
        self.y = self.y + 1
    else
        self:stop()
    end
end
function npcType:stop()
    --print(self.id, self.pathcur, self.pathnum)
    self.moving = ""
    self.ox = 0
    self.oy = 0
    self.px = self.x
    self.py = self.y
    self.pathcur = self.pathcur + 1
    if self.pathcur > self.pathnum then
        self.frame = 0
        print(self.id, "Finished path")
        self.path = nil
        self.pathstr = ""
        self.pathcur = 0
        self.pathnum = 0
        self.haspath = false
        self:wait(math.random(1,3))
    end
end

function npcType:wait(t)
    self.waittime = t
    self.doing = "wait"
end

function npcType:draw()
    if player[1].mapx == self.mapx and player[1].mapy == self.mapy then
        local x = math.floor(self.x * screen.tilesize - (self.ox * screen.tilesize) + (screen.tilesize / 2))
        local y = math.floor(self.y * screen.tilesize - (self.oy * screen.tilesize) + (screen.tilesize / 2))

        --love.graphics.setColor(0,0,255)
        --love.graphics.circle("fill", x, y, screen.tilesize / 2)

        drawpool[#drawpool+1] = {
            type = "image", x = x, y = y + 4,
            i = game.npcimg, q = game.npcquad[self.type][math.floor(self.frame)][self.facing],
            ox = 8, oy = 16 + game.map[self.x][self.y][3], who = "NPC"
        }
    end
end

function npcType:canMove(ox, oy)
    local x, y = self.x+ox, self.y+oy
    return not blocked(x, y, self.uuid)
end
