function getKeys(n)
    local keySet = {
        [1] = {
            up = "up",
            down = "down",
            left = "left",
            right = "right",
            action = " ",
        },
        [2] = {
            up = "w",
            down = "s",
            left = "a",
            right = "d",
            action = " ",
        }
    }
    return keySet[n]
end

playerType = {}

function playerType:create(...)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:init(...)
    return instance
end
function playerType:init(x, y, id, v)
    v = v or {}
    self.mapx = 1
    self.mapy = 1
    self.x = x
    self.y = y
    self.name = "Jason"
    self.speed = 8
    self.id = id
    self.uuid = createUUID()
    self.ox = 0
    self.oy = 0
    preferences.keys = getKeys(self.id)
    self.score = 0
    self.itemscollected = 0
    self.facing = "down"
    self.moving = ""
    self.frame = 0
--    self:stop()
end

function playerType:update(dt)
    if self.moving ~= "" then self.frame = (self.frame + dt * self.speed) % 2 end

    if self.ox == 0 and self.oy == 0 then
        local kUp, kDown = love.keyboard.isDown(preferences.keys.up), love.keyboard.isDown(preferences.keys.down)
        local kLeft, kRight = love.keyboard.isDown(preferences.keys.left), love.keyboard.isDown(preferences.keys.right)
        if self.ox == 0 then
            if kUp and not kDown then self.facing = "up" self:up() end
            if kDown and not kUp then self.facing = "down" self:down() end
        end
        if self.oy == 0 then
            if kLeft and not kRight then self.facing = "left" self:left() end
            if kRight and not kLeft then self.facing = "right" self:right() end
        end
    end

    if self.ox < 0 then
        self.ox = self.ox + self.speed * dt
        if self.ox > 0 then self:stop() end
    elseif self.ox > 0 then
        self.ox = self.ox - self.speed * dt
        if self.ox < 0 then self:stop() end
    end
    if self.oy < 0 then
        self.oy = self.oy + self.speed * dt
        if self.oy > 0 then self:stop() end
    elseif self.oy > 0 then
        self.oy = self.oy - self.speed * dt
        if self.oy < 0 then self:stop() end
    end

    --if self.moving ~= "" then self.facing = self.moving end
end

function playerType:left()
    if self:canMove(-1,0) then
        self.ox = -1
        self.moving = "left"
        self.x = self.x - 1
    else
        self:stop()
    end
end
function playerType:right()
    if self:canMove(1,0) then
        self.ox = 1
        self.moving = "right"
        self.x = self.x + 1
    else
        self:stop()
    end
end
function playerType:up()
    if self:canMove(0,-1) then
        self.oy = -1
        self.moving = "up"
        self.y = self.y - 1
    else
        self:stop()
    end
end
function playerType:down()
    if self:canMove(0,1) then
        self.oy = 1
        self.moving = "down"
        self.y = self.y + 1
    else
        self:stop()
    end
end
function playerType:stop()
    for _, i in pairs(item) do
        if self.x == i.x and self.y == i.y then
            i:collect(self.id)
        end
    end

    self.moving = ""
    self.ox = 0
    self.oy = 0
    if self.x == -1 then
        screen:pan("left")
    elseif self.x == 21 then
        screen:pan("right")
    elseif self.y == -1 then
        screen:pan("up")
    elseif self.y == 14 then
        screen:pan("down")
    end
end

function playerType:draw()
    local x = math.floor(self.x * screen.tilesize - (self.ox * screen.tilesize) + (screen.tilesize / 2))
    local y = math.floor(self.y * screen.tilesize - (self.oy * screen.tilesize) + (screen.tilesize / 2))

    --love.graphics.setColor(0,0,255)
    --love.graphics.circle("fill", x, y, screen.tilesize / 2)

    drawpool[#drawpool+1] = {
        type = "image", x = x, y = y + 4,
        i = game.playerimg, q = game.playerquad[math.floor(self.frame)][self.facing],
        ox = 8, oy = 16 + game.map[self.x][self.y][3], who = "player"
    }
end

function playerType:canMove(ox, oy)
    local x, y = self.x+ox, self.y+oy
    return not blocked(x, y, self.uuid)
end
