display = {}

function display:create(...)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:init(...)
    return instance
end
function display:init(w, h, v)
    v = v or {}
    self.w = w
    self.h = h
    self.tilesize = v.tilesize or 16
    self.canvas = {}
    self.panning = ""
    self.scale = love.window.getWidth() / self.w
    self.px = 0
    self.py = 0
    self.speed = .5
end
function display:addCanvas(name, w, h)
    self.canvas[name] = love.graphics.newCanvas(w, h)
    self.canvas[name]:setFilter("nearest", "nearest")
end

function display:pan(dir, mp)
    if mp == nil then mp = true end
    self.panning = dir
    self.panstage = 1
    love.graphics.setCanvas(self.canvas["pan"])
    love.graphics.draw(self.canvas["main"], 0, 0)
    love.graphics.setCanvas()

    if dir == "left" then
        self.px = 1
        player[1].mapx = player[1].mapx - 1
        if mp then
            player[1].x = 20
            player[1].ox = 0
            --player[1].moving = "left"
        end
    elseif dir == "right" then
        self.px = -1
        player[1].mapx = player[1].mapx + 1
        if mp then
            player[1].x = 0
            player[1].ox = 0
            --player[1].moving = "right"
        end
    elseif dir == "up" then
        self.py = 1
        player[1].mapy = player[1].mapy - 1
        if mp then
            player[1].y = 13
            player[1].oy = 0
            --player[1].moving = "up"
        end
    elseif dir == "down" then
        self.py = -1
        player[1].mapy = player[1].mapy + 1
        if mp then
            player[1].y = 0
            player[1].oy = 0
            --player[1].moving = "down"
        end
    end

    local e = love.filesystem.exists("maps/map_" .. player[1].mapx .. "_" .. player[1].mapy .. ".lua")
    if e then
        loadScreen(player[1].mapx, player[1].mapy)
    else
        loadScreen(-1)
    end
    drawComposite()
    love.graphics.setCanvas(self.canvas["pan2"])
    love.graphics.draw(self.canvas["main"], 0, 0)
    love.graphics.setCanvas()
end

function display:update(dt)
    if self.panstage == 1 then
        self.panstage = 2
    elseif self.panstage == 2 then
        if self.px < 0 then
            self.px = self.px + self.speed * dt
            if self.px > 0 then self:stop() end
        elseif self.px > 0 then
            self.px = self.px - self.speed * dt
            if self.px < 0 then self:stop() end
        end
        if self.py < 0 then
            self.py = self.py + self.speed * dt
            if self.py > 0 then self:stop() end
        elseif self.py > 0 then
            self.py = self.py - self.speed * dt
            if self.py < 0 then self:stop() end
        end
    elseif self.panstage == 3 then
        self.panning = ""
    end
end

function display:draw()
    love.graphics.setColor(255,255,255)
    if screen.panning ~= "" then
        local w = (screen.w+screen.tilesize) * screen.scale
        local h = (screen.h+screen.tilesize/2) * screen.scale
        local x1, y1 = 0, 0
        local x2, y2 = 0, 0
        if screen.panning == "right" then
            x2 = -(screen.px * w)
            x1 = x2-w
        elseif screen.panning == "left" then
            x2 = -(screen.px * w)
            x1 = x2+w
        elseif screen.panning == "down" then
            y2 = -(screen.py * h)
            y1 = y2-h
        elseif screen.panning == "up" then
            y2 = -(screen.py * h)
            y1 = y2+h
        end
        love.graphics.draw(screen.canvas["pan2"], x2, y2, 0, self.scale, self.scale, 8, 16)
        love.graphics.draw(screen.canvas["pan"], x1, y1, 0, self.scale, self.scale, 8, 16)
    else
        love.graphics.draw(screen.canvas["main"], 0, 0, 0, self.scale, self.scale, 8, 16)
    end
end

function display:stop()
    self.px = 0
    self.py = 0
    self.panstage = 3
end
