script = {
    running = false,
    open = "",
    run = {},
    line = 0,
    dialog = {}
}

function scriptOpen(s)
	local mFile = "scripts/" .. s .. ".lua"
	if love.filesystem.exists(mFile) then
		love.filesystem.load(mFile)()
        script.open = s
        script.line = 1
        script.running = true
	else
		error("Oops! It seems you are missing a certain script file.\nThe script \"" .. mFile .. "\" seems to be missing from its home.\nThis is not good since the game has no direction in life and must now end.")
	end

end

function scriptClose()
    script.running = false
    script.open = ""
    script.line = 0
    script.run = {}
end

function scriptUpdate(dt)
    if script.line > #script.run then
        scriptClose()
    end

    if script.running == true then
        script.run[script.line]:update(dt)
    end
end

function scriptDraw()
    if script.running == true and script.run[script.line] then
        script.run[script.line]:draw()
    end
end


function script.dialog:create(...)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:init(...)
    return instance
end
function script.dialog:init(v)
    self.text = v.text or ""
    self.font = v.font or "dialog"
    self.textspeed = v.textspeed or 50

    self.anim = 0
    self.expand = 1
    self.animspeed = v.animspeed or 10

    self.who = v.who or -1
    self.wrap = v.wrap or 200
    self.x = player[1].x * screen.tilesize
    self.y = player[1].y * screen.tilesize - (screen.tilesize / 2)
    self.padx = 8
    self.pady = 3
    self.calloutx = 0
    self.calloutside = "b"
    local width, lines = font[self.font]:getWrap(self.text, self.wrap)
    local lines = math.max(lines, 1)
    local height = font[self.font]:getHeight(self.text) * font[self.font]:getLineHeight(self.text)
    self.w = width
    self.h = math.floor(height * lines)
    self.wrap = width
    print(self.x - (self.w * .3), self.y, self.w, self.h)

    self.shadowx = 1
    self.shadowy = 2

    self.finished = false
    self.cursor = 0
    self.button = false
    self.haskeys = true
    self.pauseinput = true
end
function script.dialog:update(dt)
    if self.expand > self.anim then
        self.anim = self.anim + dt * self.animspeed
        if self.anim > 1 then self.anim = 1 end
    end
    if self.expand < self.anim then
        self.anim = self.anim - dt * self.animspeed
        if self.anim < 0 then
            self.anim = 0
            self:close()
        end
    end

    if self.anim == 1 then
        self.cursor = self.cursor + dt * self.textspeed
        if self.cursor >= #self.text then
            self.cursor = #self.text
            self.button = true
        end
    end
end
function script.dialog:draw()
    local t = string.sub(self.text, 0, math.floor(self.cursor))
    local x = (self.x-5) - (self.w * self.anim) * .3
    local y = (self.y-4) - (self.h * self.anim)
    love.graphics.setFont(font[self.font])
    love.graphics.setColor(0,0,0,100)
    roundrect("fill", x+self.shadowx, y+self.shadowy+1, (self.w+(self.padx*2)) * self.anim, (self.h+(self.pady*2)) * self.anim, 4)
    if self.anim == 1 then
        love.graphics.triangle("fill", self.x+self.shadowx, self.y+self.pady+self.shadowy, self.x+6+self.shadowx, self.y+self.pady+self.shadowy, self.x+self.shadowx, self.y+6+self.pady+self.shadowy)
    end

    love.graphics.setColor(178,220,236)
    roundrect("fill", x, y+1, (self.w+(self.padx*2)) * self.anim, (self.h+(self.pady*2)) * self.anim, 4)

    love.graphics.setColor(200,200,200)
    roundrect("fill", x, y-1, (self.w+(self.padx*2)) * self.anim, (self.h+(self.pady*2)) * self.anim, 4)

    love.graphics.setColor(255,255,255)
    roundrect("fill", x, y, (self.w+(self.padx*2)) * self.anim, (self.h+(self.pady*2)) * self.anim, 4)
    if self.anim == 1 then
        love.graphics.setColor(178,220,236)
        love.graphics.triangle("fill", self.x, self.y+self.pady-1, self.x+7, self.y+self.pady-1, self.x, self.y+7+self.pady-1)

        love.graphics.setColor(255,255,255)
        love.graphics.triangle("fill", self.x, self.y+self.pady-2, self.x+7, self.y+self.pady-2, self.x, self.y+7+self.pady-2)

        love.graphics.setColor(178,220,236)
        love.graphics.printf(t, x + self.padx, y + self.pady+1, self.wrap, "center")
        love.graphics.setColor(0,0,0)
        love.graphics.printf(t, x + self.padx, y + self.pady, self.wrap, "center")
    end
end
function script.dialog:keypressed(k, uni)
    if k == preferences.keys.action then
        if self.button == false then
            self.textspeed = #self.text * 4
        else
            self.expand = 0
        end
    end
end
function script.dialog:close()
    self.finished = true
    script.line = script.line + 1
    print(script.line)
end